
#include <unistd.h>

#include <cstdlib>
#include <iostream>
#include <iomanip>
#include <new>
#include <stdexcept>

extern void *stackBuf;

namespace {

	const size_t MB = 1024 * 1000;
}


pthread_t mi_pthread_create( void* (*f)(void*) )
{
	pthread_t tid = 0;
	pthread_attr_t threadAttrs;
	int rc = 0;

	rc = pthread_attr_init( & threadAttrs );
	if ( rc != 0 ) {
		throw std::logic_error( "error in pthread_attr_init" );
	}
	std::cerr << "pthread_attr_init done" << std::endl;

	rc = pthread_attr_setdetachstate( & threadAttrs, PTHREAD_CREATE_JOINABLE );
	if ( rc != 0 ) {
		throw std::logic_error( "error in pthread_attr_setdetachstate" );
	}
	std::cerr << "pthread_setdetachstate done" << std::endl;

	size_t stackSize = MB + sysconf( _SC_PAGESIZE );

	/*
	  stackBuf = malloc( stackSize );
	  if ( stackBuf == 0 ) {
	      throw std::bad_alloc();
	  }
	*/
	std::cerr << "allocated '" << stackSize << "' bytes for stack on "
			  << std::hex << stackBuf << std::dec << std::endl;

	memset( stackBuf, 0, stackSize );

	rc = pthread_attr_setstack( & threadAttrs, stackBuf, MB );
	if ( rc != 0 ) {
		throw std::logic_error( "error in pthread_attr_setstack" );
	}
	std::cerr << "pthread_attr_setstack done" << std::endl;


	rc = pthread_create( & tid, &threadAttrs, f, 0 );
	if ( rc != 0 ) {
		throw std::logic_error( "failed pthread_create" );
	}

	return tid;
}

